#include <string.h>
#include "error.h"
#include "cdbslib.h"
#include "loadfile.h"
#include "reffile.h"
#include "expload.h"
#include "cdbserror.h"

#define EXPLOAD_USAGE "\nUSAGE: expload input_loadfile output_loadfile expload_rulesfile"

//===========================================================================
// TASK -- Main routine for the expload task 
//
// History:
// ??/??/?? ????? ?         original
// 05/08/11 49750 MSwam     add FITS TABLE test
//===========================================================================
//
void task (int argc, char **argv)
{
    // Check number of arguments and assign to program variables
    //
    if (argc != 4) {
        fprintf(stderr, "%s\n", EXPLOAD_USAGE);
        cdbs_error (USAGE, "%d", argc-1);
    }
    char *input_loadfile    = argv[1];
    char *output_loadfile   = argv[2];
    char *expload_rulesfile = argv[3];

    // Read input loadfile. 
    //
    table *itab = read_loadfile (input_loadfile);
    if (itab == NULL)
      cdbs_error (NOLOAD, "%s", input_loadfile);

    // quit if this loadfile has already been exploaded
    //
    if (get_tab_colnum (itab, "EXPANSION_NUMBER") >= 0) {
        cdbs_error (EXPANDED, "%s", input_loadfile);
    }

    // quit if the filetype is a TABLE (tables should NOT be expanded)
    char *filename = get_tab_key(itab, "FILE_NAME");
    char actual_file_type[10];
    if (!strcmp(type_reffile (filename, actual_file_type), "table")) {
        fprintf(stderr,"ERROR: %s is a FITS table; cannot expand.\n", filename);
        cdbs_error (NOMATCH, "%s", filename);
    }

    // Create output loadfile with the same layout and header
    //
    table *otab = create_table ();
    copy_layout (itab, otab, "EXPANSION_NUMBER");
    copy_header (itab, otab);

    // Create the target and action table from the rules file.
    // The target table contains the fields that match the input table.
    // The action table contains the rows used to expand a row in the
    // input table when a match is found.
     
    table *target = create_table ();
    table *action = create_table ();

    parse_rules (expload_rulesfile, itab, target, action);

    // Use the target and action tables to expand the rows in the input
    // table into the output table
    use_rules (itab, otab, target, action);

    // Write the expanded output loadfile 
    write_loadfile (output_loadfile, otab);

    free_table (target);
    free_table (action);
}
